package org.greenstone.gsdl3.gs3build.notifier;

import java.io.*;
import java.net.*;
import java.sql.*;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.greenstone.gsdl3.gs3build.CollectionManager;
import org.greenstone.gsdl3.gs3build.database.GS3SQLConnection;

public class NotifierManager {

	public NotifierManager() {
	}
	
	public void detectEvents(CollectionManager collManager) {
		
	    System.out.println("detecting events");
	    
	    String collectionName = collManager.getCollectionName();
	    String host = "http://localhost:8080/soap/servlet/rpcrouter";
	    
		String lastBuild = dateToSqlString(collManager.getBuildDate());
		System.out.println("last build was " + lastBuild);
		Statement statement = collManager.getDatabase().createStatement();
        try {
            // detect all new documents. A document is new if and only if
            // AccessionDate >= CollectionLastRebuiltDate
        	ResultSet results = statement.executeQuery("SELECT DocID FROM document WHERE DocType != 'GSMETADATA' AND AccessionDate >= " + lastBuild);
        	notify(collectionName, host, results, collManager.getDatabase(), collManager.getBuildDate(), "new_document");
        	
        	// detect modified documents. A document is modified if and only if
        	// AccessionDate < CollectionLastRebuiltDate (ie, it is not new) and 
        	// IndexedDate >= CollectionLastRebuiltDate 
        	results = statement.executeQuery("SELECT DocID FROM document WHERE DocType != 'GSMETADATA' AND AccessionDate < " + lastBuild + " AND IndexedDate >= " + lastBuild);
        	notify(collectionName, host, results, collManager.getDatabase(), collManager.getBuildDate(), "document_modified");
        	
        	// TODO deleted docs?
        } catch (SQLException e) {
        	// TODO Auto-generated catch block
        	e.printStackTrace();
        }
	}

    /**
     * @param collName
     * @param hostURL
     * @param results
     * @param date
     * @param eventType TODO
     * @throws MalformedURLException
     * @throws IOException
     * @throws ProtocolException
     * @throws SQLException
     */
	private void notify(String collName, String hostURL, ResultSet results, GS3SQLConnection database, Date date, String eventType) {
	    try {
	        URL url = new URL("http://localhost:8080/alerting/service");
	        System.out.println("trying to send to " + url);
	        
	        while(results.next()) {
	            System.out.println(eventType + ": " + results.getString("DocID"));
	            
                String documentID = results.getString("DocID");
                
	            try {
                    HttpURLConnection conn = (HttpURLConnection) url.openConnection(); // TODO what about proxies?
                    conn.setUseCaches(false);
                    conn.setDoInput(true);
                    conn.setDoOutput(true);
                    conn.setRequestMethod("POST");
                    conn.connect();
                    // Construct data
                    OutputStreamWriter writer = new OutputStreamWriter(conn.getOutputStream());
                    
                    writer.write("action=receiveEvent&");
                    writer.write("documentID=");
                    writer.write(URLEncoder.encode(documentID, "UTF-8"));
                    writer.write("&type=");
                    writer.write(URLEncoder.encode(eventType, "UTF-8"));
                    writer.write("&collectionID=");
                    writer.write(URLEncoder.encode(collName, "UTF-8"));
                    // TODO add collection name to event
                    writer.write("&hostID=");
                    writer.write(URLEncoder.encode(hostURL, "UTF-8"));
                    writer.write("&document_title=");
                    writer.write(URLEncoder.encode(getDocumentTitle(documentID, database), "UTF-8"));
                    writer.write("&document_url=");
                    writer.write(URLEncoder.encode("http://localhost:8080/gsdl3/library?a=d&c=" + collName + "&d=" + documentID, "UTF-8"));
                    writer.write("&timestamp=");
                    writer.write(URLEncoder.encode(new SimpleDateFormat("yyyy-MM-dd HH:mm z").format(date), "UTF-8"));
                    writer.flush();
                    System.out.println(conn.getResponseCode());
                    InputStream fromReceiver = conn.getInputStream();
                    int i;
                    while ((i = fromReceiver.read()) != -1) {
                        System.out.write(i);
                    }
                    conn.disconnect();
                } catch (ProtocolException e1) {
                    // TODO Auto-generated catch block
                    e1.printStackTrace();
                } catch (UnsupportedEncodingException e1) {
                    // TODO Auto-generated catch block
                    e1.printStackTrace();
                } catch (IOException e1) {
                    // TODO Auto-generated catch block
                    e1.printStackTrace();
                }
//	            
//	            Call call = new Call();
//	            call.setTargetObjectURI("alerting");
//	            call.setMethodName("receiveEvent");
//	            call.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
//	            Vector params = new Vector();
//	            params.addElement(new Parameter("event", Map.class, event, null));
//	            call.setParams(params);
//	            try {
//                    Response resp = call.invoke(url, "");
//                    if (!resp.generatedFault()) {
//                        System.out.println("successfully posted event");
//                        return;
//                    }
//                    System.err.println("Posting event was unsuccessful:");
//                    System.err.println(resp.getFault().getFaultString());
//                    
//                    Vector entries = resp.getFault().getDetailEntries();
//                    for (Iterator i = entries.iterator(); i.hasNext(); ) {
//                      org.w3c.dom.Element entry = (org.w3c.dom.Element)i.next( );
//                      System.err.println(entry.getFirstChild().getNodeValue( ));
//                    }
//                } catch (SOAPException e1) {
//                    System.err.println("Exception while posting event: ");
//                    e1.printStackTrace();
//                }
	        }
	    } catch (SQLException e) {
	        // TODO Auto-generated catch block
	        e.printStackTrace();
	    } catch (MalformedURLException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
	}

    /**
     * @param documentID
     * @return
     * @throws SQLException
     */
    private String getDocumentTitle(String documentID, GS3SQLConnection database) {
        System.out.println("getting title for document " + documentID + " from database");
        Statement statement = database.createStatement();

        ResultSet results;
        try {
            String sqlString = "SELECT mdvalues.value " +
            		"FROM structure s JOIN divisions d ON (s.structureref = d.parentref) " +
            		"                 JOIN divisionmetarefs USING (divisionref) " + 
            		"                 JOIN metadata m USING (metaid) " +
            		"                 JOIN namespaces USING (metadataref) " + 
            		"                 JOIN mdvalues USING (namespaceref) " +
            		"  WHERE mdvalues.label = 'Title' AND s.docid = '" + documentID + "'" + 
            		"                                 AND s.structureType = 'Whole Document' " +
            		"                                 AND d.parenttype = 'Structure' " + 
            		"                                 AND m.docID = s.docid;";
            results = statement.executeQuery(sqlString);
            if (results.next()) {
                String title = results.getString("value");
                System.out.println("title is " + title);
                return title;
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return "";
    }

    /**
     * @param date
     * @return
     */
    private String dateToSqlString(Date date) {
        return new SimpleDateFormat("yyyyMMddHHmmss").format(date);
    }
}

